# -*- coding: utf-8 -*-
"""This file contains the interface for ESE database plugins."""

import os
import uuid

import pyesedb

from dfdatetime import filetime as dfdatetime_filetime

from plaso.lib import dtfabric_helper
from plaso.lib import errors
from plaso.parsers import logger
from plaso.parsers import plugins


class ESEDBPlugin(plugins.BasePlugin, dtfabric_helper.DtFabricHelper):
  """The ESE database plugin interface."""

  NAME = 'esedb_plugin'
  DATA_FORMAT = 'ESE database file'

  BINARY_DATA_COLUMN_TYPES = frozenset([
      pyesedb.column_types.BINARY_DATA,
      pyesedb.column_types.LARGE_BINARY_DATA])

  FLOATING_POINT_COLUMN_TYPES = frozenset([
      pyesedb.column_types.FLOAT_32BIT,
      pyesedb.column_types.DOUBLE_64BIT])

  INTEGER_COLUMN_TYPES = frozenset([
      pyesedb.column_types.CURRENCY,
      pyesedb.column_types.DATE_TIME,
      pyesedb.column_types.INTEGER_8BIT_UNSIGNED,
      pyesedb.column_types.INTEGER_16BIT_SIGNED,
      pyesedb.column_types.INTEGER_16BIT_UNSIGNED,
      pyesedb.column_types.INTEGER_32BIT_SIGNED,
      pyesedb.column_types.INTEGER_32BIT_UNSIGNED,
      pyesedb.column_types.INTEGER_64BIT_SIGNED])

  STRING_COLUMN_TYPES = frozenset([
      pyesedb.column_types.TEXT,
      pyesedb.column_types.LARGE_TEXT])

  # Dictionary containing a callback method per table name.
  # E.g. 'SystemIndex_0A': 'ParseSystemIndex_0A'
  REQUIRED_TABLES = {}
  OPTIONAL_TABLES = {}

  # The dtFabric definition file.
  _DEFINITION_FILE = os.path.join(
      os.path.dirname(__file__), 'types.yaml')

  def __init__(self):
    """Initializes the ESE database plugin."""
    super(ESEDBPlugin, self).__init__()
    self._tables = {}
    self._tables.update(self.REQUIRED_TABLES)
    self._tables.update(self.OPTIONAL_TABLES)

  def _ConvertValueBinaryDataToStringAscii(self, value):
    """Converts a binary data value into a string.

    Args:
      value (bytes): binary data value containing an ASCII string or None.

    Returns:
      str: string representation of binary data value or None.
    """
    if value:
      return value.decode('ascii')

    return None

  def _ConvertValueBinaryDataToStringBase16(self, value):
    """Converts a binary data value into a base-16 (hexadecimal) string.

    Args:
      value (bytes): binary data value or None.

    Returns:
      str: string representation of binary data value or None.
    """
    if value:
      return value.encode('hex')

    return None

  def _ConvertValueBinaryDataToUBInt64(self, value):
    """Converts a binary data value into an integer.

    Args:
      value (bytes): binary data value containing an unsigned 64-bit big-endian
          integer.

    Returns:
      int: integer representation of binary data value or None if value is
          not set.

    Raises:
      ParseError: if the integer value cannot be parsed.
    """
    if not value:
      return None

    integer_map = self._GetDataTypeMap('uint64be')

    try:
      return self._ReadStructureFromByteStream(value, 0, integer_map)
    except (ValueError, errors.ParseError) as exception:
      raise errors.ParseError(
          'Unable to parse integer value with error: {0!s}'.format(
              exception))

  def _ConvertValueBinaryDataToULInt64(self, value):
    """Converts a binary data value into an integer.

    Args:
      value (int): binary data value containing an unsigned 64-bit little-endian
          integer.

    Returns:
      int: integer representation of binary data value or None if value is
          not set.

    Raises:
      ParseError: if the integer value cannot be parsed.
    """
    if not value:
      return None

    integer_map = self._GetDataTypeMap('uint64le')

    try:
      return self._ReadStructureFromByteStream(value, 0, integer_map)
    except (ValueError, errors.ParseError) as exception:
      raise errors.ParseError(
          'Unable to parse integer value with error: {0!s}'.format(
              exception))

  def _GetFiletimeRecordValue(self, record_values, value_name):
    """Retrieves a FILETIME record value.

    Args:
      record_values (dict[str,object]): values per column name.
      value_name (str): name of the record value.

    Returns:
      dfdatetime.Filetime: date and time or None if not set.
    """
    filetime = record_values.get(value_name, None)
    if not filetime:
      return None

    return dfdatetime_filetime.Filetime(timestamp=filetime)

  def _GetRecordValue(self, record, value_entry):
    """Retrieves a specific value from the record.

    Args:
      record (pyesedb.record): ESE record.
      value_entry (int): value entry.

    Returns:
      object: value or None if not available.

    Raises:
      ValueError: if the value is not supported.
    """
    column_type = record.get_column_type(value_entry)
    long_value = None

    if record.is_long_value(value_entry):
      long_value = record.get_value_data_as_long_value(value_entry)

    if record.is_multi_value(value_entry):
      # TODO: implement
      raise ValueError('Multi value support not implemented yet.')

    if column_type == pyesedb.column_types.NULL:
      return None

    if column_type == pyesedb.column_types.BOOLEAN:
      if long_value:
        # TODO: implement
        raise ValueError('Long boolean value not supported.')
      return record.get_value_data_as_boolean(value_entry)

    if column_type in self.INTEGER_COLUMN_TYPES:
      if long_value:
        raise ValueError('Long integer value not supported.')
      return record.get_value_data_as_integer(value_entry)

    if column_type == pyesedb.column_types.GUID:
      if long_value:
        # TODO: implement
        raise ValueError('Long GUID value not supported.')
      value_data = record.get_value_data(value_entry)
      if value_data:
        value_data = uuid.UUID(bytes_le=value_data)
      return value_data

    if column_type in self.FLOATING_POINT_COLUMN_TYPES:
      if long_value:
        raise ValueError('Long floating point value not supported.')
      return record.get_value_data_as_floating_point(value_entry)

    if column_type in self.STRING_COLUMN_TYPES:
      if long_value:
        return long_value.get_data_as_string()
      return record.get_value_data_as_string(value_entry)

    if long_value:
      return long_value.get_data()
    return record.get_value_data(value_entry)

  def _GetRecordValues(
      self, parser_mediator, table_name, record_index, record,
      value_mappings=None):
    """Retrieves the values from the record.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      table_name (str): name of the table.
      record_index (int): ESE record index.
      record (pyesedb.record): ESE record.
      value_mappings (Optional[dict[str, str]]): value mappings, which map
          the column name to a callback method.

    Returns:
      dict[str,object]: values per column name.
    """
    record_values = {}

    for value_entry in range(0, record.number_of_values):
      if parser_mediator.abort:
        break

      column_name = record.get_column_name(value_entry)
      if column_name in record_values:
        parser_mediator.ProduceExtractionWarning(
            '[{0:s}] duplicate column: {1:s} in table: {2:s}'.format(
                self.NAME, column_name, table_name))
        continue

      value_callback = None
      if value_mappings and column_name in value_mappings:
        value_callback_method = value_mappings.get(column_name)
        if value_callback_method:
          value_callback = getattr(self, value_callback_method, None)
          if value_callback is None:
            logger.warning((
                '[{0:s}] missing value callback method: {1:s} for column: '
                '{2:s} in table: {3:s}').format(
                    self.NAME, value_callback_method, column_name, table_name))

      if value_callback:
        try:
          value_data = record.get_value_data(value_entry)
          value = value_callback(value_data)

        except Exception as exception:  # pylint: disable=broad-except
          logger.error(exception)
          value = None
          parser_mediator.ProduceExtractionWarning((
              'unable to parse value: {0:s} in record: {1:d} with callback: '
              '{2:s} in table: {3:s} with error: {4!s}').format(
                  column_name, record_index, value_callback_method, table_name,
                  exception))

      else:
        try:
          value = self._GetRecordValue(record, value_entry)
        except ValueError as exception:
          value = None
          parser_mediator.ProduceExtractionWarning((
              'unable to parse value: {0:s}  in record: {1:d} in table: {2:s} '
              'with error: {3!s}').format(
                  column_name, record_index, table_name, exception))

      record_values[column_name] = value

    return record_values

  def _ParseESEDatabase(
      self, parser_mediator, cache=None, database=None, **kwargs):
    """Extracts event objects from the database.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      cache (Optional[ESEDBCache]): cache.
      database (Optional[ESEDatabase]): ESE database.

    Raises:
      ValueError: If the database attribute is not valid.
    """
    if database is None:
      raise ValueError('Invalid database.')

    for table_name, callback_method in sorted(self._tables.items()):
      if parser_mediator.abort:
        break

      if not callback_method:
        # Table names without a callback method are allowed to improve
        # the detection of a database based on its table names.
        continue

      callback = getattr(self, callback_method, None)
      if callback is None:
        logger.warning(
            '[{0:s}] missing callback method: {1:s} for table: {2:s}'.format(
                self.NAME, callback_method, table_name))
        continue

      esedb_table = database.GetTableByName(table_name)
      if not esedb_table:
        if table_name not in self.OPTIONAL_TABLES:
          logger.warning('[{0:s}] missing table: {1:s}'.format(
              self.NAME, table_name))
        continue

      # The database is passed in case the database contains table names
      # that are assigned dynamically and cannot be defined by
      # the table name-callback mechanism.
      callback(
          parser_mediator, cache=cache, database=database, table=esedb_table,
          **kwargs)

  def CheckRequiredTables(self, database):
    """Check if the database has the minimal structure required by the plugin.

    Args:
      database (ESEDatabase): ESE database to check.

    Returns:
      bool: True if the database has the minimum tables defined by the plugin,
          or False if it does not or no required tables are defined. The
          database can have more tables than specified by the plugin and still
          return True.
    """
    if not self.REQUIRED_TABLES:
      return False

    return set(self.REQUIRED_TABLES.keys()).issubset(database.tables)

  # pylint: disable=arguments-differ
  def Process(self, parser_mediator, cache=None, database=None, **kwargs):
    """Extracts events from an ESE database.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      cache (Optional[ESEDBCache]): cache.
      database (Optional[ESEDatabase]): ESE database.

    Raises:
      ValueError: If the database argument is not valid.
    """
    if database is None:
      raise ValueError('Invalid database.')

    # This will raise if unhandled keyword arguments are passed.
    super(ESEDBPlugin, self).Process(parser_mediator)

    self._ParseESEDatabase(
        parser_mediator, cache=cache, database=database, **kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the analysis mediator."""

import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.analysis import mediator
from plaso.containers import artifacts

from tests.analysis import test_lib


class AnalysisMediatorTest(test_lib.AnalysisPluginTestCase):
  """Tests for the analysis mediator."""

  def testGetDisplayNameForPathSpec(self):
    """Tests the GetDisplayNameForPathSpec function."""
    analysis_mediator = mediator.AnalysisMediator()

    test_path = self._GetTestFilePath(['syslog.gz'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)

    expected_display_name = 'OS:{0:s}'.format(test_path)
    display_name = analysis_mediator.GetDisplayNameForPathSpec(os_path_spec)
    self.assertEqual(display_name, expected_display_name)

  def testGetUsernameForPath(self):
    """Tests the GetUsernameForPath function."""
    test_user1 = artifacts.UserAccountArtifact(
        identifier='1000', path_separator='\\',
        user_directory='C:\\Users\\testuser1',
        username='testuser1')

    analysis_mediator = mediator.AnalysisMediator(user_accounts=[test_user1])

    username = analysis_mediator.GetUsernameForPath(
        'C:\\Users\\testuser1\\Downloads')
    self.assertEqual(username, 'testuser1')

    username = analysis_mediator.GetUsernameForPath(
        'C:\\Users\\testuser2\\Downloads')
    self.assertIsNone(username)

  # TODO: add test for ProduceAnalysisReport.
  # TODO: add test for ProduceEventTag.

  def testSignalAbort(self):
    """Tests the SignalAbort function."""
    analysis_mediator = mediator.AnalysisMediator()

    analysis_mediator.SignalAbort()


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Android turbo plugin."""

import unittest

from plaso.parsers.sqlite_plugins import android_app_usage

from tests.parsers.sqlite_plugins import test_lib


class AndroidSQLiteAppUsagePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Android app_usage database plugin."""

  def testProcess(self):
    """Tests the Process function on an Android app_usage file."""
    plugin = android_app_usage.AndroidSQLiteAppUsage()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['android_app_usage'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 11545)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
      'package_name': 'com.whatsapp',
      'start_time': '2023-05-31T01:29:31.577+00:00'
    }
    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 339)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
      'package_name': 'com.spotify.music',
      'start_time': '2023-06-18T19:29:02.869+00:00'
    }
    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 8589)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Language tags helper.

For a list of language tags see:
  https://datatracker.ietf.org/doc/html/rfc5646
"""


class LanguageTagHelper(object):
  """Language tags helper."""

  _LANGUAGE_PER_TAG = {
      'af': 'Afrikaans',
      'af-ZA': 'Afrikaans, South Africa',
      'am': 'Amharic',
      'am-ET': 'Amharic, Ethiopia',
      'ar': 'Arabic',
      'ar-AE': 'Arabic, United Arab Emirates',
      'ar-BH': 'Arabic, Bahrain',
      'ar-DZ': 'Arabic, Algeria',
      'ar-EG': 'Arabic, Egypt',
      'ar-IQ': 'Arabic, Iraq',
      'ar-JO': 'Arabic, Jordan',
      'ar-KW': 'Arabic, Kuwait',
      'ar-LB': 'Arabic, Lebanon',
      'ar-LY': 'Arabic, Libyan Arab Jamahiriya',
      'ar-MA': 'Arabic, Morocco',
      'arn': 'Mapudungun',
      'arn-CL': 'Mapudungun, Chile',
      'ar-OM': 'Arabic, Oman',
      'ar-QA': 'Arabic, Qatar',
      'ar-SA': 'Arabic, Saudi Arabia',
      'ar-SY': 'Arabic, Syrian Arab Republic',
      'ar-TN': 'Arabic, Tunisia',
      'ar-YE': 'Arabic, Yemen',
      'as': 'Assamese',
      'as-IN': 'Assamese, India',
      'az': 'Azerbaijani',
      'az-Cyrl': 'Azerbaijani, Cyrillic',
      'az-Cyrl-AZ': 'Azerbaijani, Cyrillic, Azerbaijan',
      'az-Latn': 'Azerbaijani, Latin',
      'az-Latn-AZ': 'Azerbaijani, Latin, Azerbaijan',
      'ba': 'Bashkir',
      'ba-RU': 'Bashkir, Russian Federation',
      'be': 'Belarusian',
      'be-BY': 'Belarusian, Belarus',
      'bg': 'Bulgarian',
      'bg-BG': 'Bulgarian, Bulgaria',
      'bin-NG': 'Bini, Nigeria',
      'bn': 'Bengali',
      'bn-BD': 'Bengali, Bangladesh',
      'bn-IN': 'Bengali, India',
      'bo': 'Tibetan',
      'bo-BT': 'Tibetan, Bhutan',
      'bo-CN': 'Tibetan, China',
      'br': 'Breton',
      'br-FR': 'Breton, France',
      'bs': 'Bosnian',
      'bs-Cyrl': 'Bosnian, Cyrillic',
      'bs-Cyrl-BA': 'Bosnian, Cyrillic, Bosnia and Herzegovina',
      'bs-Latn': 'Bosnian, Latin',
      'bs-Latn-BA': 'Bosnian, Latin, Bosnia and Herzegovina',
      'ca': 'Catalan',
      'ca-ES': 'Catalan, Spain',
      'chr-US': 'Cherokee, United States',
      'co': 'Corsican',
      'co-FR': 'Corsican, France',
      'cs': 'Czech',
      'cs-CZ': 'Czech, Czech Republic',
      'cy': 'Welsh',
      'cy-GB': 'Welsh, United Kingdom',
      'da': 'Danish',
      'da-DK': 'Danish, Denmark',
      'de': 'German',
      'de-AT': 'German, Austria',
      'de-CH': 'German, Switzerland',
      'de-DE': 'German, Germany',
      'de-LI': 'German, Liechtenstein',
      'de-LU': 'German, Luxembourg',
      'dsb': 'Lower Sorbian',
      'dsb-DE': 'Lower Sorbian, Germany',
      'dv': 'Dhivehi',
      'dv-MV': 'Dhivehi, Maldives',
      'el': 'Modern Greek (1453 and later)',
      'el-GR': 'Modern Greek (1453-), Greece',
      'en': 'English',
      'en-AU': 'English, Australia',
      'en-BZ': 'English, Belize',
      'en-CA': 'English, Canada',
      'en-CB': 'English',
      'en-GB': 'English, United Kingdom',
      'en-HK': 'English, Hong Kong',
      'en-ID': 'English, Indonesia',
      'en-IE': 'English, Ireland',
      'en-IN': 'English, India',
      'en-JM': 'English, Jamaica',
      'en-MY': 'English, Malaysia',
      'en-NZ': 'English, New Zealand',
      'en-PH': 'English, Philippines',
      'en-SG': 'English, Singapore',
      'en-TT': 'English, Trinidad and Tobago',
      'en-US': 'English, United States',
      'en-ZA': 'English, South Africa',
      'en-ZW': 'English, Zimbabwe',
      'es': 'Spanish',
      'es-AR': 'Spanish, Argentina',
      'es-BO': 'Spanish, Bolivia',
      'es-CL': 'Spanish, Chile',
      'es-CO': 'Spanish, Colombia',
      'es-CR': 'Spanish, Costa Rica',
      'es-DO': 'Spanish, Dominican Republic',
      'es-EC': 'Spanish, Ecuador',
      'es-ES': 'Spanish, Spain',
      'es-ES_tradnl': 'Spanish',
      'es-GT': 'Spanish, Guatemala',
      'es-HN': 'Spanish, Honduras',
      'es-MX': 'Spanish, Mexico',
      'es-NI': 'Spanish, Nicaragua',
      'es-PA': 'Spanish, Panama',
      'es-PE': 'Spanish, Peru',
      'es-PR': 'Spanish, Puerto Rico',
      'es-PY': 'Spanish, Paraguay',
      'es-SV': 'Spanish, El Salvador',
      'es-US': 'Spanish, United States',
      'es-UY': 'Spanish, Uruguay',
      'es-VE': 'Spanish, Venezuela',
      'et': 'Estonian',
      'et-EE': 'Estonian, Estonia',
      'eu': 'Basque',
      'eu-ES': 'Basque, Spain',
      'fa': 'Persian',
      'fa-IR': 'Persian, Islamic Republic of Iran',
      'fi': 'Finnish',
      'fi-FI': 'Finnish, Finland',
      'fil': 'Filipino',
      'fil-PH': 'Filipino, Philippines',
      'fo': 'Faroese',
      'fo-FO': 'Faroese, Faroe Islands',
      'fr': 'French',
      'fr-BE': 'French, Belgium',
      'fr-CA': 'French, Canada',
      'fr-CG': 'French, Congo',
      'fr-CH': 'French, Switzerland',
      'fr-CI': 'French, Côte d\'Ivoire',
      'fr-CM': 'French, Cameroon',
      'fr-FR': 'French, France',
      'fr-HT': 'French, Haiti',
      'fr-LU': 'French, Luxembourg',
      'fr-MA': 'French, Morocco',
      'fr-MC': 'French, Monaco',
      'fr-ML': 'French, Mali',
      'fr-RE': 'French, Réunion',
      'fr-SN': 'French, Senegal',
      'fr-West': 'French',
      'fuv-NG': 'Nigerian Fulfulde, Nigeria',
      'fy': 'Western Frisian',
      'fy-NL': 'Western Frisian, Netherlands',
      'ga': 'Irish',
      'ga-IE': 'Irish, Ireland',
      'gaz-ET': 'West Central Oromo, Ethiopia',
      'gd': 'Scottish Gaelic',
      'gd-GB': 'Scottish Gaelic, United Kingdom',
      'gl': 'Galician',
      'gl-ES': 'Galician, Spain',
      'gn-PY': 'Guarani, Paraguay',
      'gsw': 'Swiss German',
      'gsw-FR': 'Swiss German, France',
      'gu': 'Gujarati',
      'gu-IN': 'Gujarati, India',
      'ha': 'Hausa',
      'ha-Latn': 'Hausa, Latin',
      'ha-Latn-NG': 'Hausa, Latin, Nigeria',
      'haw-US': 'Hawaiian, United States',
      'he': 'Hebrew',
      'he-IL': 'Hebrew, Israel',
      'hi': 'Hindi',
      'hi-IN': 'Hindi, India',
      'hr': 'Croatian',
      'hr-BA': 'Croatian, Bosnia and Herzegovina',
      'hr-HR': 'Croatian, Croatia',
      'hsb': 'Upper Sorbian',
      'hu': 'Hungarian',
      'hu-HU': 'Hungarian, Hungary',
      'hy': 'Armenian',
      'hy-AM': 'Armenian, Armenia',
      'ibb-NG': 'Ibibio, Nigeria',
      'id': 'Indonesian',
      'id-ID': 'Indonesian, Indonesia',
      'ig': 'Igbo',
      'ig-NG': 'Igbo, Nigeria',
      'ii': 'Sichuan Yi',
      'ii-CN': 'Sichuan Yi, China',
      'is': 'Icelandic',
      'is-IS': 'Icelandic, Iceland',
      'it': 'Italian',
      'it-CH': 'Italian, Switzerland',
      'it-IT': 'Italian, Italy',
      'iu': 'Inuktitut',
      'iu-Cans': 'Inuktitut, Unified Canadian Aboriginal Syllabics',
      'iu-Cans-CA': 'Inuktitut, Unified Canadian Aboriginal Syllabics, Canada',
      'iu-Latn': 'Inuktitut, Latin',
      'iu-Latn-CA': 'Inuktitut, Latin, Canada',
      'ja': 'Japanese',
      'ja-JP': 'Japanese, Japan',
      'ka': 'Georgian',
      'ka-GE': 'Georgian, Georgia',
      'kk': 'Kazakh',
      'kk-KZ': 'Kazakh, Kazakhstan',
      'kl': 'Kalaallisut',
      'kl-GL': 'Kalaallisut, Greenland',
      'km': 'Central Khmer',
      'km-KH': 'Central Khmer, Cambodia',
      'kn': 'Kannada',
      'kn-IN': 'Kannada, India',
      'ko': 'Korean',
      'kok': 'Konkani (macrolanguage)',
      'kok-IN': 'Konkani (macrolanguage), India',
      'ko-KR': 'Korean, Republic of Korea',
      'kr-NG': 'Kanuri, Nigeria',
      'ky': 'Kirghiz',
      'ky-KG': 'Kirghiz, Kyrgyzstan',
      'lb': 'Luxembourgish',
      'lb-LU': 'Luxembourgish, Luxembourg',
      'lo': 'Lao',
      'lo-LA': 'Lao, Lao People\'s Democratic Republic',
      'lt': 'Lithuanian',
      'lt-LT': 'Lithuanian, Lithuania',
      'lv': 'Latvian',
      'lv-LV': 'Latvian, Latvia',
      'mi': 'Maori',
      'mi-NZ': 'Maori, New Zealand',
      'mk': 'Macedonian',
      'mk-MK': 'Macedonian, The Former Yugoslav Republic of Macedonia',
      'ml': 'Malayalam',
      'ml-IN': 'Malayalam, India',
      'mn': 'Mongolian',
      'mn-Cyrl': 'Mongolian, Cyrillic',
      'mni': 'Manipuri',
      'mn-MN': 'Mongolian, Mongolia',
      'mn-Mong': 'Mongolian, Mongolian',
      'mn-Mong-CN': 'Mongolian, Mongolian, China',
      'moh': 'Mohawk',
      'moh-CA': 'Mohawk, Canada',
      'mr': 'Marathi',
      'mr-IN': 'Marathi, India',
      'ms': 'Malay (macrolanguage)',
      'ms-BN': 'Malay (macrolanguage), Brunei Darussalam',
      'ms-MY': 'Malay (macrolanguage), Malaysia',
      'mt': 'Maltese',
      'mt-MT': 'Maltese, Malta',
      'my-MM': 'Burmese, Myanmar',
      'nb': 'Norwegian Bokmål',
      'nb-NO': 'Norwegian Bokmål, Norway',
      'ne': 'Nepali',
      'ne-IN': 'Nepali, India',
      'ne-NP': 'Nepali, Nepal',
      'nl': 'Dutch',
      'nl-BE': 'Dutch, Belgium',
      'nl-NL': 'Dutch, Netherlands',
      'nn': 'Norwegian Nynorsk',
      'nn-NO': 'Norwegian Nynorsk, Norway',
      'no': 'Norwegian',
      'nso': 'Pedi',
      'nso-ZA': 'Pedi, South Africa',
      'oc': 'Occitan (post 1500)',
      'oc-FR': 'Occitan (post 1500), France',
      'or': 'Oriya',
      'or-IN': 'Oriya, India',
      'pa': 'Panjabi',
      'pa-IN': 'Panjabi, India',
      'pap-AN': 'Papiamento, Netherlands Antilles',
      'pa-PK': 'Panjabi, Pakistan',
      'pl': 'Polish',
      'pl-PL': 'Polish, Poland',
      'plt-MG': 'Plateau Malagasy, Madagascar',
      'prs': 'Dari',
      'prs-AF': 'Dari, Afghanistan',
      'ps': 'Pushto',
      'ps-AF': 'Pushto, Afghanistan',
      'pt': 'Portuguese',
      'pt-BR': 'Portuguese, Brazil',
      'pt-PT': 'Portuguese, Portugal',
      'qut': 'Guatemala',
      'qut-GT': 'Guatemala',
      'quz': 'Cusco Quechua',
      'quz-BO': 'Cusco Quechua, Bolivia',
      'quz-EC': 'Cusco Quechua, Ecuador',
      'quz-PE': 'Cusco Quechua, Peru',
      'rm': 'Romansh',
      'rm-CH': 'Romansh, Switzerland',
      'ro': 'Romanian',
      'ro-MO': 'Romanian, Macao',
      'ro-RO': 'Romanian, Romania',
      'ru': 'Russian',
      'ru-MO': 'Russian, Macao',
      'ru-RU': 'Russian, Russian Federation',
      'rw': 'Kinyarwanda',
      'rw-RW': 'Kinyarwanda, Rwanda',
      'sa': 'Sanskrit',
      'sah': 'Yakut',
      'sah-RU': 'Yakut, Russian Federation',
      'sa-IN': 'Sanskrit, India',
      'sd-IN': 'Sindhi, India',
      'sd-PK': 'Sindhi, Pakistan',
      'se': 'Northern Sami',
      'se-FI': 'Northern Sami, Finland',
      'se-NO': 'Northern Sami, Norway',
      'se-SE': 'Northern Sami, Sweden',
      'si': 'Sinhala',
      'si-LK': 'Sinhala, Sri Lanka',
      'sk': 'Slovak',
      'sk-SK': 'Slovak, Slovakia',
      'sl': 'Slovenian',
      'sl-SI': 'Slovenian, Slovenia',
      'sma': 'Southern Sami',
      'sma-NO': 'Southern Sami, Norway',
      'sma-SE': 'Southern Sami, Sweden',
      'smj': 'Lule Sami',
      'smj-NO': 'Lule Sami, Norway',
      'smj-SE': 'Lule Sami, Sweden',
      'smn': 'Inari Sami',
      'smn-FI': 'Inari Sami, Finland',
      'sms': 'Skolt Sami',
      'sms-FI': 'Skolt Sami, Finland',
      'so-SO': 'Somali, Somalia',
      'sq': 'Albanian',
      'sq-AL': 'Albanian, Albania',
      'sr': 'Serbian',
      'sr-Cyrl': 'Serbian, Cyrillic',
      'sr-Cyrl-BA': 'Serbian, Cyrillic, Bosnia and Herzegovina',
      'sr-Cyrl-CS': 'Serbian, Cyrillic, Serbia and Montenegro',
      'sr-Cyrl-ME': 'Serbian, Cyrillic, Montenegro',
      'sr-Cyrl-RS': 'Serbian, Cyrillic, Serbia',
      'sr-Latn': 'Serbian, Latin',
      'sr-Latn-BA': 'Serbian, Latin, Bosnia and Herzegovina',
      'sr-Latn-CS': 'Serbian, Latin, Serbia and Montenegro',
      'sr-Latn-ME': 'Serbian, Latin, Montenegro',
      'sr-Latn-RS': 'Serbian, Latin, Serbia',
      'st-ZA': 'Southern Sotho, South Africa',
      'sv': 'Swedish',
      'sv-FI': 'Swedish, Finland',
      'sv-SE': 'Swedish, Sweden',
      'sw': 'Swahili (macrolanguage)',
      'sw-KE': 'Swahili (macrolanguage), Kenya',
      'syr': 'Syriac',
      'syr-SY': 'Syriac, Syrian Arab Republic',
      'ta': 'Tamil',
      'ta-IN': 'Tamil, India',
      'te': 'Telugu',
      'te-IN': 'Telugu, India',
      'tg': 'Tajik',
      'tg-Cyrl': 'Tajik, Cyrillic',
      'tg-Cyrl-TJ': 'Tajik, Cyrillic, Tajikistan',
      'th': 'Thai',
      'th-TH': 'Thai, Thailand',
      'ti-ER': 'Tigrinya, Eritrea',
      'ti-ET': 'Tigrinya, Ethiopia',
      'tk': 'Turkmen',
      'tk-TM': 'Turkmen, Turkmenistan',
      'tmz': 'Tamanaku',
      'tmz-MA': 'Tamanaku, Morocco',
      'tn': 'Tswana',
      'tn-ZA': 'Tswana, South Africa',
      'tr': 'Turkish',
      'tr-TR': 'Turkish, Turkey',
      'ts-ZA': 'Tsonga, South Africa',
      'tt': 'Tatar',
      'tt-RU': 'Tatar, Russian Federation',
      'tzm': 'Central Atlas Tamazight',
      'tzm-Latn': 'Central Atlas Tamazight, Latin',
      'tzm-Latn-DZ': 'Central Atlas Tamazight, Latin, Algeria',
      'ug': 'Uighur',
      'ug-CN': 'Uighur, China',
      'uk': 'Ukrainian',
      'uk-UA': 'Ukrainian, Ukraine',
      'ur': 'Urdu',
      'ur-IN': 'Urdu, India',
      'ur-PK': 'Urdu, Pakistan',
      'uz': 'Uzbek',
      'uz-Cyrl': 'Uzbek, Cyrillic',
      'uz-Cyrl-UZ': 'Uzbek, Cyrillic, Uzbekistan',
      'uz-Latn': 'Uzbek, Latin',
      'uz-Latn-UZ': 'Uzbek, Latin, Uzbekistan',
      'ven-ZA': 'South Africa',
      'vi': 'Vietnamese',
      'vi-VN': 'Vietnamese, Viet Nam',
      'wen-DE': 'Sorbian languages, Germany',
      'wo': 'Wolof',
      'wo-SN': 'Wolof, Senegal',
      'xh': 'Xhosa',
      'xh-ZA': 'Xhosa, South Africa',
      'yo': 'Yoruba',
      'yo-NG': 'Yoruba, Nigeria',
      'zh': 'Chinese',
      'zh-CN': 'Chinese, China',
      'zh-Hans': 'Chinese, Han (Simplified variant)',
      'zh-Hant': 'Chinese, Han (Traditional variant)',
      'zh-HK': 'Chinese, Hong Kong',
      'zh-MO': 'Chinese, Macao',
      'zh-SG': 'Chinese, Singapore',
      'zh-TW': 'Chinese, Taiwan, Province of China',
      'zu': 'Zulu',
      'zu-ZA': 'Zulu, South Africa'}

  _LANGUAGE_PER_TAG_LOWER_CASE = {
      tag.lower(): language for tag, language in _LANGUAGE_PER_TAG.items()}

  @classmethod
  def GetLanguages(cls):
    """Retrieveve the language tags with their description.

    Returns:
      tuple[str, str]: lanugage tag and description.
    """
    return sorted(cls._LANGUAGE_PER_TAG.items())

  @classmethod
  def IsLanguageTag(cls, string):
    """Detemines if a string contains a valid language tag.

    Args:
      string (str): a string.

    Returns:
      bool: True if the string contains a language tag.
    """
    return string.lower() in cls._LANGUAGE_PER_TAG_LOWER_CASE



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests the for bodyfile parser."""

import unittest

from plaso.parsers import wincc

from tests.parsers import test_lib


class SIMATICTest(test_lib.ParserTestCase):
  """Tests for the SIMATIC S7 logs parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = wincc.SIMATICLogParser()
    storage_writer = self._ParseFile(
        ['wincc_simatic_s7_proto_suite.log'], parser)
    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 284)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'wincc:simatic_s7:entry',
        'creation_time': '2019-05-27T10:05:43+00:00',
        'body': ('419 INFO     | LogFileCount  : 3')
    }
    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)


class WinCCSyslogTest(test_lib.ParserTestCase):
  """Tests for the Wincc sys logs parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = wincc.WinCCSysLogParser()
    storage_writer = self._ParseFile(
        ['wincc_sys.log'], parser)
    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 768)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'wincc:sys_log:entry',
        'log_identifier': 2303,
        'creation_time': '2019-05-27T10:10:04.712',
        'event_number': 1012301,
        'log_hostname': 'BMS001',
        'source_device': 'CCWriteArchiveServer',
        'body': ('[(null) 224]failed to insert into MSARCLONG with '
                 '0x80004005(#0 \'2019-05-27 08:10:03.602\') MSG_STATE_COME')
    }
    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for Microsoft Edge load statistics plugin."""

import unittest

from plaso.parsers.sqlite_plugins import edge_load_statistics
from tests.parsers.sqlite_plugins import test_lib


class EdgeLoadStatisticsTest(test_lib.SQLitePluginTestCase):
  """Tests for edge load statistics database plugin."""

  def testProcess(self):
    """Test the Process function on a Edge Load Statistics file."""
    plugin_object = edge_load_statistics.EdgeLoadStatisticsPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['load_statistics.db'], plugin_object)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data , 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'edge:resources:load_statistics',
        'last_update': '2023-03-13T01:50:33.620419+00:00',
        'resource_hostname': 'sb.scorecardresearch.com',
        'resource_type': 4,
        'top_level_hostname': 'ntp.msn.com'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests the multi-process processing engine."""

import collections
import os
import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.containers import sessions
from plaso.lib import definitions
from plaso.engine import configurations
from plaso.multi_process import extraction_engine
from plaso.storage.sqlite import writer as sqlite_writer

from tests import test_lib as shared_test_lib


class ExtractionMultiProcessEngineTest(shared_test_lib.BaseTestCase):
  """Tests for the task-based multi-process extraction engine."""

  def testProcessSource(self):
    """Tests the PreprocessSource and ProcessSource functions."""
    test_artifacts_path = shared_test_lib.GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_engine = extraction_engine.ExtractionMultiProcessEngine(
        maximum_number_of_tasks=100)
    test_engine.BuildArtifactsRegistry(test_artifacts_path, None)

    test_file_path = self._GetTestFilePath(['ímynd.dd'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location='/',
        parent=os_path_spec)

    session = sessions.Session()

    processing_configuration = configurations.ProcessingConfiguration()
    processing_configuration.data_location = shared_test_lib.DATA_PATH
    processing_configuration.parser_filter_expression = 'filestat'
    processing_configuration.task_storage_format = (
        definitions.STORAGE_FORMAT_SQLITE)

    with shared_test_lib.TempDirectory() as temp_directory:
      temp_file = os.path.join(temp_directory, 'storage.plaso')
      storage_writer = sqlite_writer.SQLiteStorageWriter()
      storage_writer.Open(path=temp_file)

      try:
        system_configurations = test_engine.PreprocessSource(
            [source_path_spec], storage_writer)

        # The method is named ProcessSourceMulti because pylint 2.6.0 and
        # later gets confused about keyword arguments when ProcessSource
        # is used.
        processing_status = test_engine.ProcessSourceMulti(
            storage_writer, session.identifier, processing_configuration,
            system_configurations, [source_path_spec],
            storage_file_path=temp_directory)

        number_of_events = storage_writer.GetNumberOfAttributeContainers(
            'event')
        number_of_extraction_warnings = (
            storage_writer.GetNumberOfAttributeContainers(
                'extraction_warning'))
        number_of_recovery_warnings = (
            storage_writer.GetNumberOfAttributeContainers(
                'recovery_warning'))

        parsers_counter = collections.Counter({
            parser_count.name: parser_count.number_of_events
            for parser_count in storage_writer.GetAttributeContainers(
                'parser_count')})

      finally:
        storage_writer.Close()

    self.assertFalse(processing_status.aborted)

    self.assertEqual(number_of_events, 15)
    self.assertEqual(number_of_extraction_warnings, 0)
    self.assertEqual(number_of_recovery_warnings, 0)

    expected_parsers_counter = collections.Counter({
        'filestat': 15,
        'total': 15})
    self.assertEqual(parsers_counter, expected_parsers_counter)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Microsoft Internet Explorer WebCache database."""

import unittest

from plaso.parsers.esedb_plugins import msie_webcache

from tests.parsers.esedb_plugins import test_lib


class MsieWebCacheESEDBPluginTest(test_lib.ESEDBPluginTestCase):
  """Tests for the MSIE WebCache ESE database plugin."""

  # pylint: disable=protected-access

  def testConvertHeadersValues(self):
    """Tests the _ConvertHeadersValues function."""
    plugin = msie_webcache.MsieWebCacheESEDBPlugin()

    binary_value = (
        b'HTTP/1.1 200 OK\r\nContent-Type: image/png\r\n'
        b'X-Content-Type-Options: nosniff\r\nContent-Length: 2759\r\n'
        b'X-XSS-Protection: 1; mode=block\r\n'
        b'Alternate-Protocol: 80:quic\r\n\r\n')

    expected_headers_value = (
        '[HTTP/1.1 200 OK; Content-Type: image/png; '
        'X-Content-Type-Options: nosniff; Content-Length: 2759; '
        'X-XSS-Protection: 1; mode=block; '
        'Alternate-Protocol: 80:quic]')

    headers_value = plugin._ConvertHeadersValues(binary_value)
    self.assertEqual(headers_value, expected_headers_value)

  def testProcessOnDatabaseWithPartitionsTable(self):
    """Tests the Process function on database with a Partitions table."""
    plugin = msie_webcache.MsieWebCacheESEDBPlugin()
    storage_writer = self._ParseESEDBFileWithPlugin(['WebCacheV01.dat'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 341)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': '2014-05-12T07:30:25.4861987+00:00',
        'container_identifier': 1,
        'data_type': 'msie:webcache:containers',
        'directory': (
            'C:\\Users\\test\\AppData\\Local\\Microsoft\\Windows\\'
            'INetCache\\IE\\'),
        'name': 'Content',
        'set_identifier': 0}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessOnDatabaseWithPartitionsExTable(self):
    """Tests the Process function on database with a PartitionsEx table."""
    plugin = msie_webcache.MsieWebCacheESEDBPlugin()
    storage_writer = self._ParseESEDBFileWithPlugin(
        ['PartitionsEx-WebCacheV01.dat'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1143)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_count': 5,
        'cache_identifier': 0,
        'cached_file_size': 726,
        'cached_filename': 'b83d57c0[1].svg',
        'container_identifier': 14,
        'data_type': 'msie:webcache:container',
        'entry_identifier': 63,
        'modification_time': '2019-03-20T17:22:14.0000000+00:00',
        'response_headers': (
            '[HTTP/1.1 200; content-length: 726; content-type: image/svg+xml; '
            'x-cache: TCP_HIT; x-msedge-ref: Ref A: 3CD5FCBC8EAD4E0A80FA41A62'
            'FBC8CCC Ref B: PRAEDGE0910 Ref C: 2019-12-16T20:55:28Z; date: '
            'Mon, 16 Dec 2019 20:55:28 GMT]'),
        'synchronization_count': 0,
        'url': 'https://www.bing.com/rs/3R/kD/ic/878ca0cd/b83d57c0.svg'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 211)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessOnDatabaseWithCookiesExTable(self):
    """Tests the Process function on database with a CookiesEx table."""
    plugin = msie_webcache.MsieWebCacheESEDBPlugin()
    storage_writer = self._ParseESEDBFileWithPlugin(
        ['WebCacheV01_cookies.dat'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 230)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
            'cookie_hash': '5b4342ed6e2b0ae16f7e2c4c',
            'cookie_name': 'abid',
            'cookie_value': 'fcc450d1-8674-1bd3-4074-a240cff5c5b1',
            'cookie_value_raw': (
                '66636334353064312d383637342d316264332d343037342d6132343063666'
                '6356335623100'),
            'data_type': 'msie:webcache:cookie',
            'entry_identifier': 13,
            'flags': 0x80082401,
            'request_domain': 'com.associates-amazon' }

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 69)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the shared functionality for JSON output modules."""

import os
import sys
import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.lib import definitions
from plaso.output import shared_json

from tests import test_lib as shared_test_lib
from tests.containers import test_lib as containers_test_lib
from tests.output import test_lib


class SharedJSONOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests the shared functionality for JSON based output modules."""

  # pylint: disable=protected-access

  _OS_PATH_SPEC = path_spec_factory.Factory.NewPathSpec(
      dfvfs_definitions.TYPE_INDICATOR_OS, location='{0:s}{1:s}'.format(
          os.path.sep, os.path.join('cases', 'image.dd')))

  _TEST_EVENTS = [
      {'_parser_chain': 'test',
       'data_type': 'test:event',
       'hostname': 'ubuntu',
       'path_spec': path_spec_factory.Factory.NewPathSpec(
           dfvfs_definitions.TYPE_INDICATOR_TSK, inode=15,
           location='/var/log/syslog.1', parent=_OS_PATH_SPEC),
       'text': (
           'Reporter <CRON> PID: |8442| (pam_unix(cron:session): session\n '
           'closed for user root)'),
       'timestamp': '2012-06-27 18:17:01',
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN,
       'username': 'root'}]

  def testGetFieldValues(self):
    """Tests the GetFieldValues function."""
    output_mediator = self._CreateOutputMediator()

    formatters_directory_path = self._GetTestFilePath(['formatters'])
    output_mediator.ReadMessageFormattersFromDirectory(
        formatters_directory_path)

    output_module = shared_json.SharedJSONOutputModule()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    expected_timestamp = shared_test_lib.CopyTimestampFromString(
        '2012-06-27 18:17:01')

    if sys.platform.startswith('win'):
      # The dict comparison is very picky on Windows hence we
      # have to make sure the drive letter is in the same case.
      expected_os_location = os.path.abspath('\\{0:s}'.format(
          os.path.join('cases', 'image.dd')))
    else:
      expected_os_location = '{0:s}{1:s}'.format(
          os.path.sep, os.path.join('cases', 'image.dd'))

    expected_field_values = {
        '__container_type__': 'event',
        '__type__': 'AttributeContainer',
        'date_time': {
            '__class_name__': 'PosixTimeInMicroseconds',
            '__type__': 'DateTimeValues',
            'timestamp': 1340821021000000,
        },
        'data_type': 'test:event',
        'display_name': 'TSK:/var/log/syslog.1',
        'filename': '/var/log/syslog.1',
        'hostname': 'ubuntu',
        'inode': '15',
        'message': (
            'Reporter <CRON> PID: |8442| (pam_unix(cron:session): session '
            'closed for user root)'),
            'parser': 'test',
        'pathspec': {
            '__type__': 'PathSpec',
            'type_indicator': 'TSK',
            'location': '/var/log/syslog.1',
            'inode': 15,
            'parent': {
                '__type__': 'PathSpec',
                'type_indicator': 'OS',
                'location': expected_os_location,
            }
        },
        'text': (
            'Reporter <CRON> PID: |8442| (pam_unix(cron:session): '
            'session\n closed for user root)'),
        'timestamp': expected_timestamp,
        'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN,
        'username': 'root'}

    field_values = output_module.GetFieldValues(
        output_mediator, event, event_data, event_data_stream, None)

    self.assertEqual(field_values, expected_field_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The archives CLI arguments helper."""

from plaso.cli import tools
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.lib import errors


class ArchivesArgumentsHelper(interface.ArgumentsHelper):
  """Archives CLI arguments helper."""

  NAME = 'archives'
  DESCRIPTION = 'Archive command line arguments.'

  _SUPPORTED_ARCHIVE_TYPES = sorted([
      'iso9660', 'modi', 'tar', 'vhdi', 'zip'])

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        '--archives', dest='archives', type=str, action='store', default='none',
        metavar='TYPES', help=(
            'Define a list of archive and storage media image types for which '
            'to process embedded file entries, such as TAR (archive.tar) or '
            'ZIP (archive.zip). This is a comma separated list where each '
            'entry is the name of an archive type, such as "tar,zip". "all" '
            'indicates that all archive types should be enabled. "none" '
            'disables processing file entries embedded in archives. Use '
            '"--archives list" to list the available archive types. WARNING: '
            'this can make processing significantly slower.'))

  @classmethod
  def ParseOptions(cls, options, configuration_object):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      configuration_object (CLITool): object to be configured by the argument
          helper.

    Raises:
      BadConfigObject: when the configuration object is of the wrong type.
      BadConfigOption: when a configuration parameter fails validation.
    """
    if not isinstance(configuration_object, tools.CLITool):
      raise errors.BadConfigObject(
          'Configuration object is not an instance of CLITool')

    archives = getattr(options, 'archives', 'none')

    if archives == 'all':
      archives = ','.join(cls._SUPPORTED_ARCHIVE_TYPES)
    elif archives not in ('list', 'none'):
      for archive_type in archives.split(','):
        if archive_type not in cls._SUPPORTED_ARCHIVE_TYPES:
          raise errors.BadConfigOption(
               f'Unsupported archive types: {archives:s}')

    setattr(configuration_object, '_archive_types_string', archives)


manager.ArgumentHelperManager.RegisterHelper(ArchivesArgumentsHelper)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for Microsoft OneDrive log file parser."""

import unittest

from plaso.parsers import onedrive

from tests.parsers import test_lib


class OneDriveTest(test_lib.ParserTestCase):
  """Tests the OneDrive log file parser."""

  # pylint: disable=protected-access

  _AES_KEY = (
      b'\xe5\x06\x02\x00\xa3\x07\xbc\xa8\x04\x77\xf7\xf1\x33\x10\x73\xb7'
      b'\xb2\xfc\x52\x05\xcb\xb1\xdf\x3b\xd0\xc7\x98\x8f\x4e\x5f\x8c\xed')

  _RAW_PARAMETER_DATA_BYTES = (
      b'n\x01\x00\x00pnQNrDddt8-OpipvhdqEoA:\\tv99OFuY_KXMMwlQaXTSjQ\\agYrXq28'
      b'vbJnPpFW_BXEkA\\7Wkdl_bsbhXjgy3mxq5yEg\\-SHZdzOU0wA0iP5IsARudt0nSV1LvP'
      b'W6JoOIFzce_Ck\\8rCHT9NIvsVDQFxNxEN3tg\\ZKekHDq5ByJMCunDdo7QHw\\-Od38zW'
      b'spQ0v9xeoHHteiQ\\LBn1U6SqmvuW-I7vXWfWyRXY7PXItZbDbj7fy52ubH0\\3jxHZ1c8'
      b'WaPczE8XdVpS9A\\aOYl9sH9CRI3lU4XSyMpwg\\GrwrivqnY-LdeR-N5W9Hqw\\gWnvkJ'
      b'Ws277FUkYsqPUHiOedbvQSoMBRQj-u_CotLVk.cpp\x1e\x00\x00\x00CWnpConnManag'
      b'er::AsyncGetProxy\xad\x00\x00\x00\x0e\xda\x04\x80')

  def testProcessRawParameters(self):
    """Tests the _ProcessRawParameters function."""
    parser = onedrive.OneDriveLogFileParser()

    storage_writer = self._CreateStorageWriter()
    parser_mediator = self._CreateParserMediator(storage_writer)

    extracted_strings = parser._ProcessRawParameters(
        parser_mediator, self._RAW_PARAMETER_DATA_BYTES, self._AES_KEY, {})

    self.assertEqual(extracted_strings, [
        ('K:\\dbs\\sh\\odct\\1017_210557\\cmd\\2\\client\\'
         'onedrive\\Product\\wns\\trans\\WnpConnManager.cpp'),
        'CWnpConnManager::AsyncGetProxy'])

  def testRemovePKCS7Padding(self):
    """Tests the _RemovePKCS7Padding function."""
    parser = onedrive.OneDriveLogFileParser()

    unpadded_data = parser._RemovePKCS7Padding(
        b'\x08\x08\x08\x08\x08\x08\x08\x08')
    self.assertEqual(unpadded_data, b'')

    unpadded_data = parser._RemovePKCS7Padding(
        b'\x01\x02\x03\xff\x04\x04\x04\x04')
    self.assertEqual(unpadded_data, b'\x01\x02\x03\xff')

    unpadded_data = parser._RemovePKCS7Padding(
        b'\x01\x02\x03\x04\x05\x06\xff\x01')
    self.assertEqual(unpadded_data, b'\x01\x02\x03\x04\x05\x06\xff')

    unpadded_data = parser._RemovePKCS7Padding(
        b'\x01\x02\x03\x04\x05\x06\x07\xff\x08\x08\x08\x08\x08\x08\x08\x08')
    self.assertEqual(unpadded_data, b'\x01\x02\x03\x04\x05\x06\x07\xff')

    unpadded_data = parser._RemovePKCS7Padding(
        b'\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\xff\x02\x02')
    self.assertEqual(
        unpadded_data, b'\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\xff')

    with self.assertRaises(ValueError):
      parser._RemovePKCS7Padding(b'\x01\x02\x03\x04\x00\x00\x00\x00')

    with self.assertRaises(ValueError):
      parser._RemovePKCS7Padding(b'\x01\x02\x03\x04\xff\xff\xff\xff')

    with self.assertRaises(ValueError):
      parser._RemovePKCS7Padding(b'\x01\x02\x03\xff\x04\xff\x04\x04')

  def testParse(self):
    """Tests the Parse function."""
    parser = onedrive.OneDriveLogFileParser()

    storage_writer = self._ParseFile([
        'SyncEngine-2022-11-24.2341.10688.1.odlgz'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3038)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_event_data, 0)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_event_data, 0)

    expected_event_values = {
        'data_type': 'windows:onedrive:log',
        'code_filename': 'SyncServiceProxy.cpp',
        'code_function_name':
        'SyncServiceProxy::OnDownloadedSelectiveSyncEntries',
        'decoded_parameters': [
            'test document.docx', 'Document', 'B201A0178F192B2B!107',
            'B201A0178F192B2B!107.10', 'B201A0178F192B2B!101'],
        'raw_parameters':
            (b'\x10\x00\x00\x00TaxRagRight.docx\x08\x00\x00\x00Document\x14\x00'
             b'\x00\x00B201A0178F192B2B!107\x17\x00\x00\x00B201A0178F192B2B!107'
             b'.10\x14\x00\x00\x00B201A0178F192B2B!101\x00'.hex()),
        'recorded_time': '2022-11-24T23:41:48.678+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2700)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*_ coding: utf-8 -*-
"""Tests for the SCCM log text parser plugin."""

import unittest

from dfvfs.file_io import fake_file_io
from dfvfs.path import fake_path_spec
from dfvfs.resolver import context as dfvfs_context

from plaso.parsers import text_parser
from plaso.parsers.text_plugins import sccm

from tests.parsers.text_plugins import test_lib


class SCCMTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the SCCM log text parser plugin."""

  def testCheckRequiredFormat(self):
    """Tests for the CheckRequiredFormat method."""
    plugin = sccm.SCCMTextPlugin()

    resolver_context = dfvfs_context.Context()
    test_path_spec = fake_path_spec.FakePathSpec(location='/file.txt')

    file_object = fake_file_io.FakeFile(resolver_context, test_path_spec, (
        b'<![LOG[    A user is logged on to the system.]LOG]!>'
        b'<time="19:33:19.766-330" date="11-28-2014" component="AppEnforce" '
        b'context="" type="1" thread="8744" file="appprovider.cpp:2083">\n'))
    file_object.Open()

    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(None, text_reader)
    self.assertTrue(result)

    file_object = fake_file_io.FakeFile(resolver_context, test_path_spec, (
        b'(Microsoft.SoftwareCenter.Client.Data.Widget at Thingamabob)]LOG]!>'
        b'<time="10:22:50.8422964" date="1-2-2015" component="SCClient" '
        b'context="" type="0" thread="16" file="">\n'))
    file_object.Open()

    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(None, text_reader)
    self.assertFalse(result)

  def testProcess(self):
    """Tests the Process function."""
    plugin = sccm.SCCMTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['sccm_various.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 10)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test log entry with milliseconds precision and time zone offset.
    # time="19:33:19.766-330" date="11-28-2014"
    expected_event_values = {
        'component': 'AppEnforce',
        'data_type': 'sccm_log:entry',
        'text': ('+++ Starting Install enforcement for App DT "Application '
                 'Foo Version 2.2" ApplicationDeliveryType - ScopeId_AD87A846-'
                 'E6A5-4088-875F-066CF1082D30/DeploymentType_14a11199-ee14-'
                 '4c06-a5a7-68eadf501337, Revision - 10, ContentPath - '
                 'C:\\Windows\\ccmcache\\u, Execution Context - System'),
        'written_time': '2014-11-28T19:33:19.766-06:30'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Test log entry with 100ns precision and time zone offset.
    # time="10:22:50.8422964" date="1-2-2015"
    expected_event_values = {
        'component': 'SCClient',
        'data_type': 'sccm_log:entry',
        'written_time': '2015-01-02T10:22:50.873496+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 3)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The parser mediator."""

import collections
import datetime
import time

from plaso.containers import artifacts
from plaso.containers import events
from plaso.containers import warnings
from plaso.engine import path_helper
from plaso.engine import profilers
from plaso.helpers import language_tags
from plaso.helpers.windows import languages


class ParserMediator(object):
  """Parser mediator.

  Attributes:
    last_activity_timestamp (int): timestamp received that indicates the last
        time activity was observed. The last activity timestamp is updated
        when the mediator produces an attribute container, such as an event
        source. This timestamp is used by the multi processing worker process
        to indicate the last time the worker was known to be active. This
        information is then used by the foreman to detect workers that are
        not responding (stalled).
    parsers_counter (collections.Counter): number of events per parser or
        parser plugin.
    registry_find_specs (list[dfwinreg.FindSpec]): Windows Registry find
        specifications.
  """

  _DEFAULT_CODE_PAGE = 'cp1252'

  _DEFAULT_LANGUAGE_TAG = 'en-US'.lower()

  # LCID 0x0409 is en-US.
  _DEFAULT_LCID = 0x0409

  def __init__(
      self, registry_find_specs=None, resolver_context=None,
      system_configurations=None):
    """Initializes a parser mediator.

    Args:
      registry_find_specs (Optional[list[dfwinreg.FindSpec]]): Windows Registry
          find specifications.
      resolver_context (Optional[dfvfs.Context]): resolver context.
      system_configurations (Optional[list[SystemConfigurationArtifact]]):
          system configurations.
    """
    super(ParserMediator, self).__init__()
    self._abort = False
    self._cached_parser_chain = None
    self._environment_variables_per_path_spec = None
    self._event_data_stream = None
    self._event_data_stream_identifier = None
    self._extract_winevt_resources = True
    self._extract_winreg_binary_values = False
    self._file_entry = None
    self._format_checks_cpu_time_profiler = None
    self._language_tag = None
    self._last_event_data_hash = None
    self._last_event_data_identifier = None
    self._lcid = None
    self._number_of_event_data = 0
    self._number_of_event_sources = 0
    self._number_of_extraction_warnings = 0
    self._number_of_recovery_warnings = 0
    self._parser_chain_components = []
    self._parsers_cpu_time_profiler = None
    self._parsers_memory_profiler = None
    self._preferred_code_page = None
    self._process_information = None
    self._resolver_context = resolver_context
    self._storage_writer = None
    self._temporary_directory = None
    self._windows_event_log_providers = None
    self._windows_event_log_providers_per_filename = None
    self._windows_event_log_providers_per_path = None

    self.registry_find_specs = registry_find_specs
    self.last_activity_timestamp = 0.0
    self.parsers_counter = collections.Counter()

    self._CreateEnvironmentVariablesPerPathSpec(system_configurations)

  @property
  def abort(self):
    """bool: True if parsing should be aborted."""
    return self._abort

  @property
  def extract_winevt_resources(self):
    """bool: extract Windows EventLog resources."""
    return self._extract_winevt_resources

  @property
  def extract_winreg_binary_values(self):
    """bool: extract Windows Registry binary values."""
    return self._extract_winreg_binary_values

  @property
  def number_of_produced_event_data(self):
    """int: number of produced event data."""
    return self._number_of_event_data

  @property
  def number_of_produced_event_sources(self):
    """int: number of produced event sources."""
    return self._number_of_event_sources

  @property
  def number_of_produced_extraction_warnings(self):
    """int: number of produced extraction warnings."""
    return self._number_of_extraction_warnings

  @property
  def resolver_context(self):
    """dfvfs.Context: resolver context."""
    return self._resolver_context

  @property
  def temporary_directory(self):
    """str: path of the directory for temporary files."""
    return self._temporary_directory

  def _CreateEnvironmentVariablesPerPathSpec(self, system_configurations):
    """Creates the environment variables per path specification lookup table.

    Args:
      system_configurations (list[SystemConfigurationArtifact]): system
          configurations.
    """
    self._environment_variables_per_path_spec = {}
    for system_configuration in system_configurations or []:
      if system_configuration.environment_variables:
        for path_spec in system_configuration.path_specs:
          if path_spec.parent:
            self._environment_variables_per_path_spec[path_spec.parent] = (
                system_configuration.environment_variables)

  def _CreateEventLogMessageFileLookupTables(self, environment_variables):
    """Creates the Windows EventLog message file lookup tables.

    Args:
      environment_variables (list[EnvironmentVariableArtifact]): environment
          variables.
    """
    self._windows_event_log_providers_per_filename = {}
    self._windows_event_log_providers_per_path = {}

    for provider in self._windows_event_log_providers:
      self._FillEventLogMessageFileLookupTablesWithMessageFiles(
          environment_variables, provider, provider.event_message_files)

      self._FillEventLogMessageFileLookupTablesWithMessageFiles(
          environment_variables, provider, provider.parameter_message_files)

  def _FillEventLogMessageFileLookupTablesWithMessageFiles(
      self, environment_variables, event_log_provider, message_files):
    """Fills the Windows EventLog message file lookup tables with message files.

    Args:
      environment_variables (list[EnvironmentVariableArtifact]): environment
          variables.
      event_log_provider (WindowsEventLogProviderArtifact): Windows EventLog
          provider.
      message_files (list[str]): message file paths.
    """
    for windows_path in message_files or []:
      path, filename = path_helper.PathHelper.GetWindowsSystemPath(
          windows_path, environment_variables)
      path = path.lower()
      filename = filename.lower()

      # Use the path prefix as the key to handle language specific EventLog
      # message files.
      if path not in self._windows_event_log_providers_per_path:
        self._windows_event_log_providers_per_path[path] = {}

      # Note that multiple providers can share EventLog message files.
      self._windows_event_log_providers_per_filename[filename] = (
          event_log_provider)
      self._windows_event_log_providers_per_path[path][filename] = (
          event_log_provider)

  def _GetEnvironmentVariablesByPathSpec(self, path_spec):
    """Retrieves the environment variables for a specific path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      list[EnvironmentVariableArtifact]: environment variables.
    """
    if not path_spec or not path_spec.parent:
      return None

    return self._environment_variables_per_path_spec.get(path_spec.parent, None)

  def AddDateLessLogHelper(self, date_less_log_helper):
    """Adds a date-less log helper.

    Args:
      date_less_log_helper (DateLessLogHelper): date-less log helper.
    """
    if self._event_data_stream_identifier:
      date_less_log_helper.SetEventDataStreamIdentifier(
          self._event_data_stream_identifier)

    self._storage_writer.AddAttributeContainer(date_less_log_helper)

  def AddWindowsEventLogMessageFile(self, message_file):
    """Adds a Windows EventLog message file.

    Args:
      message_file (WindowsEventLogMessageFileArtifact): Windows EventLog
          message file.
    """
    self._storage_writer.AddAttributeContainer(message_file)

  def AddWindowsEventLogMessageString(self, message_string):
    """Adds a Windows EventLog message string.

    Args:
      message_string (WindowsEventLogMessageStringArtifact): Windows EventLog
          message string.
    """
    self._storage_writer.AddAttributeContainer(message_string)

  def AddWindowsWevtTemplateEvent(self, event_definition):
    """Adds a Windows WEVT_TEMPLATE event definition.

    Args:
      event_definition (WindowsWevtTemplateEvent): Windows WEVT_TEMPLATE event
          definition.
    """
    self._storage_writer.AddAttributeContainer(event_definition)

  def AppendToParserChain(self, name):
    """Adds a parser or parser plugin to the parser chain.

    Args:
      name (str): name of a parser or parser plugin.
    """
    self._cached_parser_chain = None
    self._parser_chain_components.append(name)

  def ClearParserChain(self):
    """Clears the parser chain."""
    self._cached_parser_chain = None
    self._parser_chain_components = []

  def ExpandWindowsPath(self, path):
    """Expands a Windows path containing environment variables.

    Args:
      path (str): Windows path with environment variables.

    Returns:
      str: expanded Windows path.
    """
    path_spec = getattr(self._file_entry, 'path_spec', None)
    environment_variables = self._GetEnvironmentVariablesByPathSpec(path_spec)
    return path_helper.PathHelper.ExpandWindowsPath(path, environment_variables)

  def GetCodePage(self):
    """Retrieves the code page related to the file entry.

    Returns:
      str: code page.
    """
    path_spec = getattr(self._file_entry, 'path_spec', None)
    if path_spec:
      # TODO: determine code page from system_configurations.
      pass

    return self._preferred_code_page or self._DEFAULT_CODE_PAGE

  def GetCurrentYear(self):
    """Retrieves current year.

    Returns:
      int: the current year.
    """
    datetime_object = datetime.datetime.now()
    return datetime_object.year

  def GetDisplayName(self, file_entry=None):
    """Retrieves the display name for a file entry.

    Args:
      file_entry (Optional[dfvfs.FileEntry]): file entry object, where None
          will use the active file entry.

    Returns:
      str: human readable string that describes the path to the file entry.

    Raises:
      ValueError: if the file entry is missing.
    """
    if file_entry is None:
      file_entry = self._file_entry

    if file_entry is None:
      raise ValueError('Missing file entry')

    path_spec = getattr(file_entry, 'path_spec', None)

    relative_path = path_helper.PathHelper.GetRelativePathForPathSpec(path_spec)
    if not relative_path:
      return file_entry.name

    return path_helper.PathHelper.GetDisplayNameForPathSpec(path_spec)

  def GetDisplayNameForPathSpec(self, path_spec):
    """Retrieves the display name for a path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      str: human readable version of the path specification.
    """
    return path_helper.PathHelper.GetDisplayNameForPathSpec(path_spec)

  def GetFileEntry(self):
    """Retrieves the active file entry.

    Returns:
      dfvfs.FileEntry: file entry or None if not available.
    """
    return self._file_entry

  def GetFilename(self):
    """Retrieves the name of the active file entry.

    Returns:
      str: name of the active file entry or None.
    """
    if not self._file_entry:
      return None

    data_stream = getattr(self._file_entry.path_spec, 'data_stream', None)
    if data_stream:
      return f'{self._file_entry.name:s}:{data_stream:s}'

    return self._file_entry.name

  def GetLanguageTag(self):
    """Retrieves the language tag related to the file entry.

    Returns:
      str: code page.
    """
    path_spec = getattr(self._file_entry, 'path_spec', None)
    if path_spec:
      # TODO: determine language tag from system_configurations.
      pass

    return self._language_tag or self._DEFAULT_LANGUAGE_TAG

  def GetParserChain(self):
    """Retrieves the current parser chain.

    Returns:
      str: parser chain.
    """
    if not self._cached_parser_chain:
      self._cached_parser_chain = '/'.join(self._parser_chain_components)
    return self._cached_parser_chain

  def GetRelativePath(self):
    """Retrieves the relative path of the current file entry.

    Returns:
      str: relative path of the current file entry or None if no current
          file entry.
    """
    path_spec = getattr(self._file_entry, 'path_spec', None)
    if not path_spec:
      return None

    return path_helper.PathHelper.GetRelativePathForPathSpec(path_spec)

  def GetRelativePathForPathSpec(self, path_spec):
    """Retrieves the relative path for a path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      str: relative path of the path specification.
    """
    return path_helper.PathHelper.GetRelativePathForPathSpec(path_spec)

  def GetWindowsEventLogMessageFile(self):
    """Retrieves the Windows EventLog message file for a specific path.

    Returns:
      WindowsEventLogMessageFileArtifact: Windows EventLog message file or None
          if no current file entry or no Windows EventLog message file was
          found.
    """
    path_spec = getattr(self._file_entry, 'path_spec', None)
    if not path_spec:
      return None

    if (self._windows_event_log_providers_per_filename is None and
        self._windows_event_log_providers_per_path is None):
      # TODO: improve this by building lookup tables per volume path spec
      environment_variables = self._GetEnvironmentVariablesByPathSpec(path_spec)
      if not environment_variables:
        return None

      self._CreateEventLogMessageFileLookupTables(environment_variables)

    relative_path = path_helper.PathHelper.GetRelativePathForPathSpec(path_spec)
    lookup_path = relative_path.lower()

    path_segment_separator = path_helper.PathHelper.GetPathSegmentSeparator(
        path_spec)

    lookup_path, _, lookup_filename = lookup_path.rpartition(
        path_segment_separator)

    # Language specific EventLog message file paths contain a language tag
    # such as "en-US".
    base_lookup_path, _, last_path_segment = lookup_path.rpartition(
        path_segment_separator)
    if language_tags.LanguageTagHelper.IsLanguageTag(last_path_segment):
      lookup_path = base_lookup_path

    message_file = None

    if (lookup_path == '\\windows\\systemresources' and
        lookup_filename[-4:] == '.mun'):
      lookup_filename = lookup_filename[:-4]

      provider = self._windows_event_log_providers_per_filename.get(
          lookup_filename, None)
      if provider:
        windows_path = '\\'.join(['', 'Windows', 'System32', lookup_filename])
        message_file = artifacts.WindowsEventLogMessageFileArtifact(
            path=relative_path, windows_path=windows_path)

    else:
      providers_per_filename = self._windows_event_log_providers_per_path.get(
          lookup_path, {})

      for filename, provider in providers_per_filename.items():
        if lookup_filename in (filename, f'{filename:s}.mui'):
          windows_path = '\\'.join([lookup_path, filename])
          message_file = artifacts.WindowsEventLogMessageFileArtifact(
              path=relative_path, windows_path=windows_path)
          break

    return message_file

  def PopFromParserChain(self):
    """Removes the last added parser or parser plugin from the parser chain."""
    self._cached_parser_chain = None
    self._parser_chain_components.pop()

  def ProduceEventData(self, event_data):
    """Produces event data.

    Args:
      event_data (EventData): event data.

    Raises:
      RuntimeError: when storage writer is not set.
    """
    if not self._storage_writer:
      raise RuntimeError('Storage writer not set.')

    parser_chain = getattr(event_data, '_parser_chain', None)
    if not parser_chain:
      parser_chain = self.GetParserChain()
      setattr(event_data, '_parser_chain', parser_chain)

    if self._event_data_stream_identifier:
      event_data.SetEventDataStreamIdentifier(
          self._event_data_stream_identifier)

    event_values_hash = events.CalculateEventValuesHash(
        event_data, self._event_data_stream)
    setattr(event_data, '_event_values_hash', event_values_hash)

    self._storage_writer.AddAttributeContainer(event_data)
    self._number_of_event_data += 1

    self.last_activity_timestamp = time.time()

  def ProduceEventDataStream(self, event_data_stream):
    """Produces an event data stream.

    Args:
      event_data_stream (EventDataStream): an event data stream or None if no
          event data stream is needed.

    Raises:
      RuntimeError: when storage writer is not set.
    """
    if not self._storage_writer:
      raise RuntimeError('Storage writer not set.')

    if not event_data_stream:
      self._event_data_stream = None
      self._event_data_stream_identifier = None
    else:
      if not event_data_stream.path_spec:
        event_data_stream.path_spec = getattr(
            self._file_entry, 'path_spec', None)

      self._storage_writer.AddAttributeContainer(event_data_stream)

      self._event_data_stream = event_data_stream
      self._event_data_stream_identifier = event_data_stream.GetIdentifier()

    self.last_activity_timestamp = time.time()

  def ProduceEventSource(self, event_source):
    """Produces an event source.

    Args:
      event_source (EventSource): an event source.

    Raises:
      RuntimeError: when storage writer is not set.
    """
    if not self._storage_writer:
      raise RuntimeError('Storage writer not set.')

    self._storage_writer.AddAttributeContainer(event_source)
    self._number_of_event_sources += 1

    self.last_activity_timestamp = time.time()

  def ProduceExtractionWarning(self, message, path_spec=None):
    """Produces an extraction warning.

    Args:
      message (str): message of the warning.
      path_spec (Optional[dfvfs.PathSpec]): path specification, where None
          will use the path specification of current file entry set in
          the mediator.

    Raises:
      RuntimeError: when storage writer is not set.
    """
    if not self._storage_writer:
      raise RuntimeError('Storage writer not set.')

    if not path_spec and self._file_entry:
      path_spec = self._file_entry.path_spec

    parser_chain = self.GetParserChain()
    warning = warnings.ExtractionWarning(
        message=message, parser_chain=parser_chain, path_spec=path_spec)
    self._storage_writer.AddAttributeContainer(warning)
    self._number_of_extraction_warnings += 1

    self.last_activity_timestamp = time.time()

  def ProduceRecoveryWarning(self, message, path_spec=None):
    """Produces a recovery warning.

    Args:
      message (str): message of the warning.
      path_spec (Optional[dfvfs.PathSpec]): path specification, where None
          will use the path specification of current file entry set in
          the mediator.

    Raises:
      RuntimeError: when storage writer is not set.
    """
    if not self._storage_writer:
      raise RuntimeError('Storage writer not set.')

    if not path_spec and self._file_entry:
      path_spec = self._file_entry.path_spec

    parser_chain = self.GetParserChain()
    warning = warnings.RecoveryWarning(
        message=message, parser_chain=parser_chain, path_spec=path_spec)
    self._storage_writer.AddAttributeContainer(warning)
    self._number_of_recovery_warnings += 1

    self.last_activity_timestamp = time.time()

  def SetWindowsEventLogProviders(self, windows_event_log_providers):
    """Sets the Windows EventLog providers.

    Rags:
      windows_event_log_providers (list[WindowsEventLogProviderArtifact]):
          Windows EventLog providers.
    """
    self._windows_event_log_providers = windows_event_log_providers

  def ResetFileEntry(self):
    """Resets the active file entry."""
    self._file_entry = None

  def SampleFormatCheckStartTiming(self, parser_name):
    """Starts timing a CPU time sample for profiling.

    Args:
      parser_name (str): name of the parser.
    """
    if self._format_checks_cpu_time_profiler:
      self._format_checks_cpu_time_profiler.StartTiming(parser_name)

  def SampleFormatCheckStopTiming(self, parser_name):
    """Stops timing a CPU time sample for profiling.

    Args:
      parser_name (str): name of the parser.
    """
    if self._format_checks_cpu_time_profiler:
      self._format_checks_cpu_time_profiler.StopTiming(parser_name)

  def SampleMemoryUsage(self, parser_name):
    """Takes a sample of the memory usage for profiling.

    Args:
      parser_name (str): name of the parser.
    """
    if self._parsers_memory_profiler:
      used_memory = self._process_information.GetUsedMemory() or 0
      self._parsers_memory_profiler.Sample(parser_name, used_memory)

  def SampleStartTiming(self, parser_name):
    """Starts timing a CPU time sample for profiling.

    Args:
      parser_name (str): name of the parser.
    """
    if self._parsers_cpu_time_profiler:
      self._parsers_cpu_time_profiler.StartTiming(parser_name)

  def SampleStopTiming(self, parser_name):
    """Stops timing a CPU time sample for profiling.

    Args:
      parser_name (str): name of the parser.
    """
    if self._parsers_cpu_time_profiler:
      self._parsers_cpu_time_profiler.StopTiming(parser_name)

  def SetExtractWinEvtResources(self, extract_winevt_resources):
    """Sets value to extract Windows EventLog resources.

    Args:
      extract_winevt_resources (bool): True if Windows EventLog resources
          should be extracted.
    """
    self._extract_winevt_resources = extract_winevt_resources

  def SetExtractWinRegBinaryValues(self, extract_winreg_binary_values):
    """Sets value to extract Windows Registry binary values.

    Args:
      extract_winreg_binary_values (bool): True if Windows Registry binary
          values should be extracted.
    """
    self._extract_winreg_binary_values = extract_winreg_binary_values

  def SetFileEntry(self, file_entry):
    """Sets the active file entry.

    Args:
      file_entry (dfvfs.FileEntry): file entry.
    """
    self._event_data_stream = None
    self._event_data_stream_identifier = None
    self._file_entry = file_entry

  def SetPreferredCodepage(self, code_page):
    """Sets the preferred code page.

    Args:
      code_page (str): code page.
    """
    if code_page:
      code_page = code_page.lower()
    self._preferred_code_page = code_page

  def SetPreferredLanguage(self, language_tag):
    """Sets the preferred language.

    Args:
      language_tag (str): language tag such as "en-US" for US English or
          "is-IS" for Icelandic or None if the language determined by
          preprocessing or the default should be used.

    Raises:
      ValueError: if the language tag is not a string type or no LCID can
          be determined that corresponds with the language tag.
    """
    lcid = None
    if language_tag:
      lcid = languages.WindowsLanguageHelper.GetLCIDForLanguageTag(language_tag)
      if not lcid:
        raise ValueError(f'No LCID found for language tag: {language_tag:s}.')

      language_tag = language_tag.lower()

    self._language_tag = language_tag
    self._lcid = lcid

  def SetStorageWriter(self, storage_writer):
    """Sets the storage writer.

    Args:
      storage_writer (StorageWriter): storage writer.
    """
    self._storage_writer = storage_writer

    # Reset the last event data information. Each storage file should
    # contain event data for their events.
    self._last_event_data_hash = None

  def SetTemporaryDirectory(self, temporary_directory):
    """Sets the directory to store temporary files.

    Args:
      temporary_directory (str): path of the directory to store temporary files.
    """
    self._temporary_directory = temporary_directory

  def SignalAbort(self):
    """Signals the parsers to abort."""
    self._abort = True

  def StartProfiling(self, configuration, identifier, process_information):
    """Starts profiling.

    Args:
      configuration (ProfilingConfiguration): profiling configuration.
      identifier (str): identifier of the profiling session used to create
          the sample filename.
      process_information (ProcessInfo): process information.
    """
    if not configuration:
      return

    if configuration.HaveProfileFormatChecks():
      identifier = f'{identifier:s}-format_checks'

      self._format_checks_cpu_time_profiler = profilers.CPUTimeProfiler(
          identifier, configuration)
      self._format_checks_cpu_time_profiler.Start()

    if configuration.HaveProfileParsers():
      identifier = f'{identifier:s}-parsers'

      self._parsers_cpu_time_profiler = profilers.CPUTimeProfiler(
          identifier, configuration)
      self._parsers_cpu_time_profiler.Start()

      self._parsers_memory_profiler = profilers.MemoryProfiler(
          identifier, configuration)
      self._parsers_memory_profiler.Start()

    self._process_information = process_information

  def StopProfiling(self):
    """Stops profiling."""
    if self._format_checks_cpu_time_profiler:
      self._format_checks_cpu_time_profiler.Stop()
      self._format_checks_cpu_time_profiler = None

    if self._parsers_cpu_time_profiler:
      self._parsers_cpu_time_profiler.Stop()
      self._parsers_cpu_time_profiler = None

    if self._parsers_memory_profiler:
      self._parsers_memory_profiler.Stop()
      self._parsers_memory_profiler = None

    self._process_information = None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the default Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.parsers.winreg_plugins import default

from tests.parsers.winreg_plugins import test_lib


class TestDefaultRegistry(test_lib.RegistryPluginTestCase):
  """Tests for the default Windows Registry plugin."""

  def _CreateTestKey(self):
    """Creates Registry keys and values for testing.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromDateTimeString('2012-08-28 09:23:49.002031')
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'TimeZoneInformation', key_path_prefix='HKEY_CURRENT_MACHINE\\Software',
        last_written_time=filetime.timestamp, offset=1456,
        relative_key_path='Microsoft\\Some Windows\\InterestingApp\\MRU')

    value_data = 'acb'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'MRUList', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=123)
    registry_key.AddValue(registry_value)

    value_data = 'Some random text here'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'a', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=1892)
    registry_key.AddValue(registry_value)

    value_data = 'c:/evil.exe'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'b', data=value_data, data_type=dfwinreg_definitions.REG_BINARY,
        offset=612)
    registry_key.AddValue(registry_value)

    value_data = 'C:/looks_legit.exe'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'c', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=1001)
    registry_key.AddValue(registry_value)

    return registry_key

  def testProcess(self):
    """Tests the Process function."""
    registry_key = self._CreateTestKey()

    plugin = default.DefaultPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_key_path = (
        'HKEY_CURRENT_MACHINE\\Software\\Microsoft\\Some Windows\\'
        'InterestingApp\\MRU')

    expected_event_values = {
        'data_type': 'windows:registry:key_value',
        'key_path': expected_key_path,
        'last_written_time': '2012-08-28T09:23:49.0020310+00:00',
        'values': [
            ('MRUList', 'REG_SZ', 'acb'),
            ('a', 'REG_SZ', 'Some random text here'),
            ('b', 'REG_BINARY', '(22 bytes)'),
            ('c', 'REG_SZ', 'C:/looks_legit.exe')]}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Windows EventLog providers helper."""


class WindowsEventLogProvidersHelper(object):
  """Windows EventLog providers helper."""

  def _GetNormalizedPath(self, path):
    """Retrieves a normalized variant of a path.

    Args:
      path (str): path of a message file.

    Returns:
      str: normalized path of a message file.
    """
    path_segments = path.split('\\')
    filename = path_segments.pop()

    if path_segments:
      # Check if the first path segment is a drive letter or "%SystemDrive%".
      first_path_segment = path_segments[0].lower()
      if ((len(first_path_segment) == 2 and first_path_segment[1:] == ':') or
          first_path_segment == '%systemdrive%'):
        path_segments[0] = ''

    path_segments_lower = [
        path_segment.lower() for path_segment in path_segments]

    if not path_segments_lower:
      # If the path is a filename assume the file is stored in:
      # "%SystemRoot%\System32".
      path_segments = ['%SystemRoot%', 'System32']

    elif path_segments_lower[0] in ('system32', '$(runtime.system32)'):
      # Note that the path can be relative so if it starts with "System32"
      # asume this represents "%SystemRoot%\System32".
      path_segments = ['%SystemRoot%', 'System32'] + path_segments[1:]

    elif path_segments_lower[0] in (
        '%systemroot%', '%windir%', '$(runtime.windows)'):
      path_segments = ['%SystemRoot%'] + path_segments[1:]

    # Check if path starts with "\SystemRoot\", "\Windows\" or "\WinNT\" for
    # example: "\SystemRoot\system32\drivers\SerCx.sys"
    elif (len(path_segments_lower) > 1 and not path_segments_lower[0] and
          path_segments_lower[1] in ('systemroot', 'windows', 'winnt')):
      path_segments = ['%SystemRoot%'] + path_segments[2:]

    path_segments.append(filename)
    return '\\'.join(path_segments) or '\\'

  def Merge(self, first_event_log_provider, second_event_log_provider):
    """Merges the information of the second Event Log provider into the first.

    Args:
      first_event_log_provider (EventLogProvider): first Event Log provider.
      second_event_log_provider (EventLogProvider): second Event Log provider.
    """
    if not first_event_log_provider.identifier:
      first_event_log_provider.identifier = second_event_log_provider.identifier
    elif (first_event_log_provider.identifier !=
          second_event_log_provider.identifier):
      first_event_log_provider.additional_identifier = (
          second_event_log_provider.identifier)

    for log_source in second_event_log_provider.log_sources:
      log_source = log_source.lower()
      if log_source not in first_event_log_provider.log_sources:
        first_event_log_provider.log_sources.append(log_source)

    for log_type in second_event_log_provider.log_types:
      log_type = log_type.lower()
      if log_type not in first_event_log_provider.log_types:
        first_event_log_provider.log_types.append(log_type)

    for path in second_event_log_provider.category_message_files:
      path = self._GetNormalizedPath(path)
      if path not in first_event_log_provider.category_message_files:
        first_event_log_provider.category_message_files.append(path)

    for path in second_event_log_provider.event_message_files:
      path = self._GetNormalizedPath(path)
      if path not in first_event_log_provider.event_message_files:
        first_event_log_provider.event_message_files.append(path)

    for path in second_event_log_provider.parameter_message_files:
      path = self._GetNormalizedPath(path)
      if path not in first_event_log_provider.parameter_message_files:
        first_event_log_provider.parameter_message_files.append(path)

  def NormalizeMessageFiles(self, event_log_provider):
    """Normalizes the message files.

    Args:
      event_log_provider (EventLogProvider): Event Log provider.
    """
    event_log_provider.category_message_files = [
        self._GetNormalizedPath(path)
        for path in event_log_provider.category_message_files]

    event_log_provider.event_message_files = [
        self._GetNormalizedPath(path)
        for path in event_log_provider.event_message_files]

    event_log_provider.parameter_message_files = [
        self._GetNormalizedPath(path)
        for path in event_log_provider.parameter_message_files]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Skype main.db history database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import skype

from tests.parsers.sqlite_plugins import test_lib


class SkypePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Skype main.db history database plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = skype.SkypePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['skype_main.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 20)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test transfer file entry.
    expected_event_values = {
        'accept_time': None,
        'data_type': 'skype:event:transferfile',
        'end_time': None,
        'destination': 'european.bbq.competitor <European BBQ>',
        'source': 'gen.beringer <Gen Beringer>',
        'start_time': '2013-10-24T21:49:32+00:00',
        'transfer_status': 2,
        'transferred_filename': 'secret-project.pdf'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 17)
    self.CheckEventData(event_data, expected_event_values)

    # Test SMS entry.
    expected_event_values = {
        'data_type': 'skype:event:sms',
        'number': '+34123456789',
        'recorded_time': '2013-07-01T22:14:22+00:00',
        'text': ('If you want I can copy some documents for you, if you can '
                 'pay it... ;)')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 16)
    self.CheckEventData(event_data, expected_event_values)

    # Test chat entry.
    expected_event_values = {
        'data_type': 'skype:event:chat',
        'from_account': 'Gen Beringer <gen.beringer>',
        'recorded_time': '2013-07-30T21:27:11+00:00',
        'text': 'need to know if you got it this time.',
        'title': 'European Competitor | need to know if you got it..',
        'to_account': 'european.bbq.competitor'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

    # Test call entry.
    expected_event_values = {
        'attempt_time': '2013-07-01T22:12:17+00:00',
        'data_type': 'skype:event:call',
        'dst_call': 'european.bbq.competitor',
        'end_time': '2013-07-01T22:23:03+00:00',
        'src_call': 'gen.beringer',
        'start_time': '2013-07-01T22:12:17+00:00',
        'user_start_call': False,
        'video_conference': False}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 19)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text parser plugin for Windows SetupAPI log files.

Also see:
  https://learn.microsoft.com/en-us/windows-hardware/drivers/install/setupapi-text-logs
"""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class SetupAPILogEventData(events.EventData):
  """SetupAPI log event data.

  Attributes:
    end_time (dfdatetime.DateTimeValues): date and time the end of the log
        entry was added.
    entry_type (str): log entry type, for examaple "Device Install -
        PCI\\VEN_104C&DEV_8019&SUBSYS_8010104C&REV_00\\3&61aaa01&0&38" or
        "Sysprep Respecialize - {804b345a-ffd7-854c-a1b5-ca9598907846}".
    exit_status (str): the exit status of the logged operation.
    start_time (dfdatetime.DateTimeValues): date and time the start of
        the log entry was added.
  """

  DATA_TYPE = 'setupapi:log:line'

  def __init__(self):
    """Initializes event data."""
    super(SetupAPILogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.end_time = None
    self.entry_type = None
    self.exit_status = None
    self.start_time = None


class SetupAPILogTextPlugin(interface.TextPlugin):
  """Text parser plugin for Windows SetupAPI log files."""

  NAME = 'setupapi'
  DATA_FORMAT = 'Windows SetupAPI log file'

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _THREE_DIGITS = pyparsing.Word(pyparsing.nums, exact=3).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _FOUR_DIGITS = pyparsing.Word(pyparsing.nums, exact=4).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  # Date and time values are formatted as: 2015/11/22 17:59:28.110
  _DATE_TIME = (
      _FOUR_DIGITS + pyparsing.Suppress('/') +
      _TWO_DIGITS + pyparsing.Suppress('/') + _TWO_DIGITS +
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS + pyparsing.Suppress(':') + _TWO_DIGITS +
      pyparsing.Word('.,', exact=1).suppress() + _THREE_DIGITS)

  # pylint: disable=line-too-long
  # See https://docs.microsoft.com/en-us/windows-hardware/drivers/install/format-of-a-text-log-header
  # pylint: enable=line-too-long
  _DEVICE_INSTALL_LOG_LINE = (
      pyparsing.Literal('[Device Install Log]') + _END_OF_LINE)

  # Using a regular expression here is faster. Note that pyparsing 2 does not
  # properly handle leading whitespace.
  _HEADER_ENTRY_LINE = pyparsing.Regex(
       r'(Architecture|OS Version|ProductType|Service Pack|Suite) = .*\n')

  _BEGIN_LOG_LINE = pyparsing.Literal('[BeginLog]') + _END_OF_LINE

  # pylint: disable=line-too-long
  # See https://docs.microsoft.com/en-us/windows-hardware/drivers/install/format-of-a-text-log-section-header
  # pylint: enable=line-too-long

  # Using a regular expression here is faster.
  _SECTION_HEADER_LINE = pyparsing.Regex(r'>>>  \[(?P<entry_type>[^\]]+)\]\n')

  # Using a regular expression here is faster.
  _SECTION_START_LINE = pyparsing.Regex(
      r'>>>  Section start (?P<start_time>[0-9]{4}/[0-9]{2}/[0-9]{2} '
      r'[0-9]{2}:[0-9]{2}:[0-9]{2}\.[0-9]{3})\n')

  # pylint: disable=line-too-long
  # See https://docs.microsoft.com/en-us/windows-hardware/drivers/install/format-of-a-text-log-section-footer
  # pylint: enable=line-too-long

  # Using a regular expression here is faster.
  _SECTION_END_LINE = pyparsing.Regex(
      r'<<<  Section end (?P<end_time>[0-9]{4}/[0-9]{2}/[0-9]{2} '
      r'[0-9]{2}:[0-9]{2}:[0-9]{2}\.[0-9]{3})\n')

  # Using a regular expression here is faster.
  _EXIT_STATUS_LINE = pyparsing.Regex(
      r'<<<  \[Exit status: (?P<exit_status>[^\]]+)\]\n')

  # pylint: disable=line-too-long
  # See https://learn.microsoft.com/en-us/windows-hardware/drivers/install/format-of-a-text-log-section-body
  # and https://docs.microsoft.com/en-us/windows-hardware/drivers/install/format-of-log-entries-that-are-not-part-of-a-text-log-section
  # pylint: enable=line-too-long

  # Note that undocumented event catagegories have been observed, such as:
  # "cmd:", "idb:" and "pol:".

  # Using a regular expression here is faster. Note that pyparsing 2 does not
  # properly handle leading whitespace.
  _LOG_ENTRY_LINE = pyparsing.Regex(
      r'(|\. |!    |!!!  )[A-Za-z\.]{2,3}: .{0,336}\n')

  # Undocumented observed lines.

  # Using a regular expression here is faster.
  _BOOT_SESSION_LINE = pyparsing.Regex(
      r'[Boot Session: [0-9]{4}/[0-9]{2}/[0-9]{2} '
      r'[0-9]{2}:[0-9]{2}:[0-9]{2}\.[0-9]{3}]\n')

  _HEADER_GRAMMAR = (
      _DEVICE_INSTALL_LOG_LINE +
      pyparsing.OneOrMore(_HEADER_ENTRY_LINE) +
      _BEGIN_LOG_LINE)

  _LINE_STRUCTURES = [
      ('boot_session_line', _BOOT_SESSION_LINE),
      ('exit_status_line', _EXIT_STATUS_LINE),
      ('log_entry_line', _LOG_ENTRY_LINE),
      ('section_end_line', _SECTION_END_LINE),
      ('section_header_line', _SECTION_HEADER_LINE),
      ('section_start_line', _SECTION_START_LINE)]

  VERIFICATION_GRAMMAR = _DEVICE_INSTALL_LOG_LINE

  def __init__(self):
    """Initializes a text parser plugin."""
    super(SetupAPILogTextPlugin, self).__init__()
    self._event_data = None

  def _ParseHeader(self, parser_mediator, text_reader):
    """Parses a text-log file header.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Raises:
      ParseError: when the header cannot be parsed.
    """
    try:
      structure_generator = self._HEADER_GRAMMAR.scan_string(
          text_reader.lines, max_matches=1)
      structure, start, end = next(structure_generator)

    except StopIteration:
      structure = None

    except pyparsing.ParseException as exception:
      raise errors.ParseError(exception)

    if not structure or start != 0:
      raise errors.ParseError('No match found.')

    text_reader.SkipAhead(end)

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    if key == 'section_header_line':
      self._event_data = SetupAPILogEventData()
      self._event_data.entry_type = self._GetValueFromStructure(
          structure, 'entry_type')

    elif key == 'section_start_line':
      start_time_structure = self._GetValueFromStructure(
          structure, 'start_time')

      try:
        time_elements_structure = self._DATE_TIME.parse_string(
            start_time_structure)
      except pyparsing.ParseException as exception:
        raise errors.ParseError(
            'Unable to parse start time with error: {0!s}'.format(exception))

      self._event_data.start_time = self._ParseTimeElements(
          time_elements_structure)

    elif key == 'section_end_line':
      end_time_structure = self._GetValueFromStructure(
          structure, 'end_time')

      try:
        time_elements_structure = self._DATE_TIME.parse_string(
            end_time_structure)
      except pyparsing.ParseException as exception:
        raise errors.ParseError(
            'Unable to parse end time with error: {0!s}'.format(exception))

      self._event_data.end_time = self._ParseTimeElements(
          time_elements_structure)

    elif key == 'exit_status_line':
      self._event_data.exit_status = self._GetValueFromStructure(
          structure, 'exit_status')

      parser_mediator.ProduceEventData(self._event_data)

      self._ResetState()

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements of a log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      (year, month, day_of_month, hours, minutes, seconds, milliseconds) = (
          time_elements_structure)

      # Ensure time_elements_tuple is not a pyparsing.ParseResults otherwise
      # copy.deepcopy() of the dfDateTime object will fail on Python 3.8 with:
      # "TypeError: 'str' object is not callable" due to pyparsing.ParseResults
      # overriding __getattr__ with a function that returns an empty string
      # when named token does not exist.
      time_elements_tuple = (
          year, month, day_of_month, hours, minutes, seconds, milliseconds)

      date_time = dfdatetime_time_elements.TimeElementsInMilliseconds(
          time_elements_tuple=time_elements_tuple)

      # SetupAPI logs store date and time values in local time.
      date_time.is_local_time = True

      return date_time

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def _ResetState(self):
    """Resets stored values."""
    self._event_data = None

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct plugin, False otherwise.
    """
    try:
      self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    self._ResetState()

    return True


text_parser.TextLogParser.RegisterPlugin(SetupAPILogTextPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Safari Downloads plist plugin."""

import unittest

from plaso.parsers.plist_plugins import safari_downloads

from tests.parsers.plist_plugins import test_lib


class SafariDownloadsPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the Safari Downloads plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'Downloads.plist'

    plugin = safari_downloads.SafariDownloadsPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'safari:downloads:entry',
        'end_time': '2022-12-14T19:05:30.620490+00:00',
        'full_path': (
            '~/Downloads/paint.net.4.3.12.install.anycpu.web.zip.download/'
            'paint.net.4.3.12.install.anycpu.web.zip'),
        'received_bytes': 826337,
        'remove_on_completion': False,
        'start_time': '2022-12-14T19:05:26.862793+00:00',
        'total_bytes': 826337,
        'url': (
            'https://www.dotpdn.com/files/'
            'paint.net.4.3.12.install.anycpu.web.zip')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the timezone Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.parsers.winreg_plugins import timezone

from tests.parsers.winreg_plugins import test_lib


class WinRegTimezonePluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the timezone Windows Registry plugin."""

  def _CreateTestKey(self):
    """Creates Registry keys and values for testing.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromDateTimeString('2013-01-30 10:47:57')
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'TimeZoneInformation', key_path_prefix='HKEY_LOCAL_MACHINE\\System',
        last_written_time=filetime.timestamp, offset=153,
        relative_key_path='ControlSet001\\Control\\TimeZoneInformation')

    value_data = 'C:\\Downloads\\plaso-static.rar'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        '1', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=612)
    registry_key.AddValue(registry_value)

    value_data = b'\xff\xff\xff\xc4'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ActiveTimeBias', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    registry_key.AddValue(registry_value)

    value_data = b'\xff\xff\xff\xc4'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Bias', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    registry_key.AddValue(registry_value)

    value_data = b'\xff\xff\xff\xc4'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DaylightBias', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    registry_key.AddValue(registry_value)

    value_data = '@tzres.dll,-321'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DaylightName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = (
        b'\x00\x00\x03\x00\x05\x00\x02\x00\x00\x00\x00\x00\x00\x00\x00\x00')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DaylightStart', data=value_data,
        data_type=dfwinreg_definitions.REG_BINARY)
    registry_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DynamicDaylightTimeDisabled', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    registry_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'StandardBias', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    registry_key.AddValue(registry_value)

    value_data = '@tzres.dll,-322'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'StandardName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = (
        b'\x00\x00\x0A\x00\x05\x00\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'StandardStart', data=value_data,
        data_type=dfwinreg_definitions.REG_BINARY)
    registry_key.AddValue(registry_value)

    value_data = 'W. Europe Standard Time'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'TimeZoneKeyName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    return registry_key

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = timezone.WinRegTimezonePlugin()

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\System', (
        'ControlSet001\\Control\\TimeZoneInformation'))

    self._AssertNotFiltersOnKeyPath(
        plugin, 'HKEY_LOCAL_MACHINE\\System', 'Bogus')

  def testProcessMock(self):
    """Tests the Process function on created key."""
    registry_key = self._CreateTestKey()

    plugin = timezone.WinRegTimezonePlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_key_path = (
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control\\'
        'TimeZoneInformation')

    expected_event_values = {
        'configuration': (
            'ActiveTimeBias: -60 '
            'Bias: -60 '
            'DaylightBias: -60 '
            'DaylightName: @tzres.dll,-321 '
            'DynamicDaylightTimeDisabled: 0 '
            'StandardBias: 0 '
            'StandardName: @tzres.dll,-322 '
            'TimeZoneKeyName: W. Europe Standard Time'),
        'data_type': 'windows:registry:timezone',
        'key_path': expected_key_path,
        'last_written_time': '2013-01-30T10:47:57.0000000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessFile(self):
    """Tests the Process function on registry file."""
    test_file_entry = self._GetTestFileEntry(['SYSTEM'])
    key_path = (
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control\\'
        'TimeZoneInformation')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = timezone.WinRegTimezonePlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'configuration': (
            'ActiveTimeBias: 240 '
            'Bias: 300 '
            'DaylightBias: -60 '
            'DaylightName: @tzres.dll,-111 '
            'DynamicDaylightTimeDisabled: 0 '
            'StandardBias: 0 '
            'StandardName: @tzres.dll,-112 '
            'TimeZoneKeyName: Eastern Standard Time'),
        'data_type': 'windows:registry:timezone',
        'key_path': key_path,
        'last_written_time': '2012-03-11T07:00:00.0006424+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows firewall log text parser plugin."""

import unittest

from plaso.parsers.text_plugins import winfirewall

from tests.parsers.text_plugins import test_lib


class WinFirewallLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the Windows firewall log text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = winfirewall.WinFirewallLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['windows_firewall.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 15)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:firewall_log:entry',
        'destination_ip': '123.156.78.90',
        'destination_port': 1774,
        'last_written_time': '2005-04-11T08:06:26',
        'source_ip': '123.45.78.90',
        'source_port': 80,
        'packet_size': 576,
        'tcp_ack': 987654321,
        'tcp_flags': 'A',
        'tcp_sequence_number': 123456789,
        'tcp_window_size': 12345}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 7)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()
